@@ -6,7 +6,7 @@ from threading import Timer
 
 import requests
 from requests.adapters import HTTPAdapter
-from requests.packages.urllib3.util.retry import Retry
+from urllib3.util import Retry
 from ..version import __version__
 
 instances = []  # For keeping track of running class instances
@@ -44,17 +44,32 @@ class WatchTowerHandler(logging.Handler):
     A logging handler to send events to a Watch Tower instance
     """
 
-    def __init__(self, beam_id, host=DEFAULT_HOST, use_fallback=True, fallback_host=None, token=None, debug=False, flush_interval=5.0,
-                 force_keep_ahead=False, protocol='https',
-                 queue_size=DEFAULT_QUEUE_SIZE,
-                 retry_backoff=2.0, retry_count=2, timeout=1.0, verify=True, version=__version__, dev=False):
+    def __init__(self,
+                 beam_id: str,
+                 token: (str, None) = None,
+                 host: str = DEFAULT_HOST,
+                 use_fallback: bool = True,
+                 fallback_host: (str, None) = None,
+                 dev: bool = False,
+                 debug: bool = False,
+                 flush_interval: float = 5.0,
+                 force_keep_ahead: bool = False,
+                 protocol: str = 'https',
+                 queue_size: int = DEFAULT_QUEUE_SIZE,
+                 retry_backoff: float = 2.0,
+                 retry_count: int = 2,
+                 timeout: float = 1.0,
+                 verify: bool = True,
+                 version: str = __version__):
         """
         Args:
-            host (str): The Watch Tower host (should include port if necessary)
-            fallback_host (str): The Watchtower fallback host, defaults to fallback.<host>
+            beam_id (str): The id of the Watchtower beam to send the events to
             token (str): Authentication token
+            host (str): The Watchtower host (should include port if necessary)
+            fallback_host (str): The Watchtower fallback host, defaults to fallback.<host>
+            dev (bool): Whether to send the dev flag with all event
             debug (bool): Whether to print debug console messages
-            flush_interval (float): How often to push events to Watch Tower in seconds
+            flush_interval (float): How often to push events to Watchtower in seconds
             force_keep_ahead (bool): Sleep instead of dropping logs when queue fills
             protocol (str): The web protocol to use
             queue_size (int): The max number of logs to queue, set to 0 for no max
@@ -62,6 +77,7 @@ class WatchTowerHandler(logging.Handler):
             retry_count (int): The number of times to retry a failed request
             timeout (float): The time to wait for a response from Watch Tower
             verify (bool): Whether to perform ssl certificate validation
+            version (str): Version string to specify the watchtower-logging library version
         """
 
         global instances
@@ -92,21 +108,12 @@ class WatchTowerHandler(logging.Handler):
         self.version = version
         self.dev = dev
 
-        self.url = '{protocol}://{host}/api/beams/{beam_id}'.format(protocol=self.protocol,
-                                                                    host=self.host,
-                                                                    beam_id=self.beam_id)
-        self.fallback_url = '{protocol}://{host}/api/beams/{beam_id}'.format(protocol=self.protocol,
-                                                                             host=self.fallback_host,
-                                                                             beam_id=self.beam_id)
-
-
         # Keep ahead depends on queue size, so cannot be 0
         if self.force_keep_ahead and not self.max_queue_size:
             self.write_log('Cannot keep ahead of unbound queue, using default queue size')
             self.max_queue_size = DEFAULT_QUEUE_SIZE
 
         self.write_debug_log('Starting debug mode')
-
         self.write_debug_log('Preparing to override loggers')
 
         # prevent infinite recursion by silencing requests and urllib3 loggers
@@ -126,8 +133,7 @@ class WatchTowerHandler(logging.Handler):
         # Set up automatic retry with back-off
         self.write_debug_log('Preparing to create a Requests session')
         retry = Retry(total=self.retry_count,
-                      backoff_factor=self.retry_backoff,
-                      method_whitelist=False,  # Retry for any HTTP verb
+                      backoff_factor=self.retry_backoff,  # Retry for any HTTP verb
                       status_forcelist=[500, 502, 503, 504])
         self.session.mount(self.protocol + '://', HTTPAdapter(max_retries=retry))
 
@@ -135,7 +141,17 @@ class WatchTowerHandler(logging.Handler):
 
         self.write_debug_log('Class initialize complete')
 
-    def emit(self, record):
+    def build_endpoint(self,
+                       host: str):
+
+        return '{protocol}://{host}/api/beams/{beam_id}'.format(
+            protocol=self.protocol,
+            host=host,
+            beam_id=self.beam_id)
+
+    def emit(self,
+             record):
+
         self.write_debug_log("emit() called")
 
         try:
@@ -163,6 +179,7 @@ class WatchTowerHandler(logging.Handler):
             self.write_log('Log queue full; log data will be dropped.')
 
     def close(self):
+
         self.shutdown()
         logging.Handler.close(self)
 
@@ -171,6 +188,7 @@ class WatchTowerHandler(logging.Handler):
     #
 
     def start_worker_thread(self):
+
         # Start a worker thread responsible for sending logs
         if self.flush_interval > 0:
             self.write_debug_log('Preparing to spin off first worker thread Timer')
@@ -178,21 +196,28 @@ class WatchTowerHandler(logging.Handler):
             self.timer.daemon = True  # Auto-kill thread if main process exits
             self.timer.start()
 
-    def write_log(self, log_message):
+    def write_log(self,
+                  log_message: str):
+
         print('[WatchTowerHandler] ' + log_message)
 
-    def write_debug_log(self, log_message):
+    def write_debug_log(self,
+                        log_message: str):
+
         if self.debug:
             print('[WatchTowerHandler DEBUG] ' + log_message)
 
-    def format_record(self, record):
+    def format_record(self,
+                      record):
 
         self.write_debug_log('format_record() called')
         record.dev = self.dev
         return self.format(record)
 
 
-    def _wt_worker(self, payload=None):
+    def _wt_worker(self,
+                   payload: (str, None) = None):
+
         self.write_debug_log("_wt_worker() called")
         if self.flush_interval > 0:
             # Stop the timer. Happens automatically if this is called
@@ -282,9 +307,11 @@ class WatchTowerHandler(logging.Handler):
                 self.timer.daemon = True  # Auto-kill thread if main process exits
                 self.timer.start()
                 self.write_debug_log("Timer thread scheduled")
+
         self.processing_payload = False
 
     def empty_queue(self):
+
         if len(self.queue) == 0:
             self.write_debug_log("Queue was empty")
             return True
@@ -305,11 +332,13 @@ class WatchTowerHandler(logging.Handler):
         return len(self.queue) > 0
 
     def force_flush(self):
+
         self.write_debug_log("Force flush requested")
         self._wt_worker()
         self.wait_until_empty()  # guarantees queue is emptied
 
     def shutdown(self):
+
         self.write_debug_log("Immediate shutdown requested")
 
         # Only initiate shutdown once
@@ -328,6 +357,7 @@ class WatchTowerHandler(logging.Handler):
         self.wait_until_empty()  # guarantees queue is emptied before exit
 
     def wait_until_empty(self):
+
         self.write_debug_log("Waiting until queue empty")
         while len(self.queue) > 0 or self.processing_payload:
             self.write_debug_log("Current queue size: " + str(len(self.queue)))
@@ -335,8 +365,20 @@ class WatchTowerHandler(logging.Handler):
 
     @property
     def alt_flush_interval(self):
+
         return min(1.0, self.flush_interval / 2)
 
     @property
     def user_agent(self):
-        return 'watchtower-logging-python/{version}'.format(version=self.version)
\ No newline at end of file
+
+        return 'watchtower-logging-python/{version}'.format(version=self.version)
+
+    @property
+    def url(self):
+
+        return self.build_endpoint(host=self.host)
+
+    @property
+    def fallback_url(self):
+
+        return self.build_endpoint(host=self.fallback_host)
\ No newline at end of file
