@@ -1,9 +1,11 @@
 import logging
+import inspect
 import os
 import types
 from watchtower_logging.watchtower_handler import WatchTowerHandler
 from watchtower_logging.exceptions import loggingException
 from watchtower_logging.version import __version__
+from watchtower_logging.utils import monitor_func, send_alert, get_env
 from pythonjsonlogger import jsonlogger
 import random
 import string
@@ -13,6 +15,7 @@ import traceback
 import sys
 import hashlib
 import json
+import time
 
 DONE_LEVEL_NUM = logging.INFO + 4
 START_LEVEL_NUM = logging.INFO + 2
@@ -38,9 +41,36 @@ class logLevels(object):
     ERROR = logging.ERROR
     CRITICAL = logging.CRITICAL
 
-
 def logaction(self, msg, lvl_num, args, data=None, **kws):
+
     if self.isEnabledFor(lvl_num):
+        
+        try:
+            
+            if not sys.exc_info()[2] is None:
+                
+                tb = sys.exc_info()[2]
+                tb_info = traceback.extract_tb(tb)
+                last_call_stack = tb_info[-1]  # Get the last call stack
+
+                frame_info = {
+                    'filename': last_call_stack.filename,
+                    'lineno': last_call_stack.lineno,
+                    'function_name': last_call_stack.name,
+                }
+
+            else:
+
+                frame_info = inspect.getframeinfo(inspect.currentframe().f_back.f_back)
+                frame_info = {
+                    'filename': frame_info.filename,
+                    'lineno': frame_info.lineno,
+                    'function': frame_info.function,
+                }
+
+        except Exception:
+
+            frame_info = None
 
         if hasattr(self, '_default_data') and isinstance(self._default_data, dict):
             if data is None:
@@ -53,7 +83,7 @@ def logaction(self, msg, lvl_num, args, data=None, **kws):
                 if not 'traceback' in data:
                     if not sys.exc_info()[2] is None:
                         data['traceback'] = traceback.format_exc()
-
+        
             if 'extra' in kws:
                 if 'data' in kws['extra']:
                     raise loggingException('Duplicate `data` key, please merge the data you are passing with the entry in `extra`.')
@@ -80,6 +110,11 @@ def logaction(self, msg, lvl_num, args, data=None, **kws):
         if not 'beam_id' in extra:
             extra['beam_id'] = self.beam_id
 
+        if not hasattr(self, 'env_send_time') or time.time() - self.env_send_time > self.env_interval:
+            if not 'env__' in extra:
+                extra['env__'] = self.env
+            self.env_send_time = time.time()
+
         # adding deduplication-id
         if self.dedup and not DEDUP_ID_KEY in extra:
             if not self.dedup_keys is None and isinstance(self.dedup_keys, (list, tuple, set)):
@@ -96,6 +131,10 @@ def logaction(self, msg, lvl_num, args, data=None, **kws):
 
             extra[self.dedup_id_key] = dedup_id
 
+        if lvl_num >= self.lvl_frame_info:
+            if not 'frame__' in extra:
+                extra['frame__'] = frame_info
+
         # Yes, logger takes its '*args' as 'args'.
         self._log(lvl_num, msg, args, extra=extra, **kws)
 
@@ -135,6 +174,10 @@ def setDefaultData(self, data, overwrite=False):
     else:
         self._default_data = data
 
+def setReturnWhenException(self, return_object=None):
+
+    self.return_when_exception = return_object
+
 class CustomJsonFormatter(jsonlogger.JsonFormatter):
 
     converter = datetime.datetime.utcfromtimestamp
@@ -158,11 +201,13 @@ class CustomJsonFormatter(jsonlogger.JsonFormatter):
 
 def getLogger(beam_id, name=None, execution_id=None, token=None, host=None, protocol=None, dev=False, default_data=None,
               level=logLevels.START, debug=False, path=None, console=False, send=True, use_threading=None, dedup=True,
-              dedup_keys=None, retry_count=3, catch_all=True, dedup_id_key=None, use_fallback=True):
+              dedup_keys=None, retry_count=3, catch_all=True, dedup_id_key=None, use_fallback=True, lvl_frame_info=0,
+              env_interval=600):
 
     # create logger
-    name = name or os.environ.get('FUNCTION_NAME', 'watchtower-logging')
+    name = name or os.environ.get('K_SERVICE') or os.environ.get('FUNCTION_NAME', 'watchtower-logging')
     logger = logging.getLogger(name)
+    logger.lvl_frame_info = lvl_frame_info
     logger.logaction = types.MethodType(logaction, logger)
     logger.done = types.MethodType(done, logger)
     logger.start = types.MethodType(start, logger)
@@ -172,13 +217,21 @@ def getLogger(beam_id, name=None, execution_id=None, token=None, host=None, prot
     logger.error = types.MethodType(error, logger)
     logger.critical = types.MethodType(critical, logger)
     logger.setExecutionId = types.MethodType(setExecutionId, logger)
+    logger.monitor_func = types.MethodType(monitor_func, logger)
     logger.setLevel(level)
     logger.setDefaultData = types.MethodType(setDefaultData, logger)
+    logger.setReturnWhenException = types.MethodType(setReturnWhenException, logger)
+    logger.send_alert = types.MethodType(send_alert, logger)
     logger.dedup = dedup
     logger.dedup_keys = dedup_keys
     logger.dedup_id_key = dedup_id_key or DEDUP_ID_KEY
     logger.beam_id = beam_id
-
+    logger.dev = dev
+    logger.host = host
+    logger.env = get_env()
+    logger.env_interval = env_interval
+    
+    logger.init_default_data = default_data
     if default_data:
         logger.setDefaultData(data=default_data)
 
